local addon = ...

-- functions
local strsplit = string.split
local unpack = unpack
local C_ChallengeMode = C_ChallengeMode
local C_ChallengeMode_GetMapInfo= C_ChallengeMode.GetMapInfo
local GetAutoCompleteRealms = GetAutoCompleteRealms
local GetRealmName = GetRealmName
local UnitFactionGroup = UnitFactionGroup
-- constants
local NUM_BAG_SLOTS = NUM_BAG_SLOTS
-- frames
local GameTooltip = GameTooltip
-- libs
local DataStore = DataStore
local LibStub = LibStub

local THIS_ACCOUNT = "Default"
local THIS_REALM = GetRealmName()
local THIS_FACTION = UnitFactionGroup("player")

local function GetCharacterDisplayName(realm, faction, character)
    local name = DataStore:GetCharacterName(character)

    if realm == THIS_REALM then
        if faction ~= THIS_FACTION then
            return string.format("%s|r %s", DataStore:GetColoredCharacterFaction(character), DataStore:GetColoredCharacterName(character))
        end

        return DataStore:GetColoredCharacterName(character)
    end

    name = string.format("%s-%s", name, realm)

    local classColour = DataStore:GetCharacterClassColor(character)
    if faction ~= THIS_FACTION then
        return string.format("%s|r %s%s|r", DataStore:GetColoredCharacterFaction(character), classColour, name)
    end

    return string.format("%s%s|r", classColour, name)
end

local function GetKeystoneData(itemLink)
    local itemString = itemLink:match("|Hitem:([0-9:]+)|h(%b[])|h")
    local info = {strsplit(":", itemString)}

    local instanceMapId = tonumber(info[14])
    local level = tonumber(info[15])

    -- wtf
    local isDepleted = 1 ~= tonumber(level > 9 and info[19] or (level > 6 and info[18] or (level > 4 and info[17 or info[16]])))

    return instanceMapId, level, isDepleted
end

local function GetKeystoneForCharacter(character)
    local containers = DataStore:GetContainers(character)

    for bagId = 0, NUM_BAG_SLOTS do
        local bag = containers["Bag" .. bagId]
        for i, slotItemId in pairs(bag.ids) do
            if slotItemId == 138019 then
                return bag.links[i]
            end
        end
    end

    return nil
end

local function GetCharactersWithKeystones()
    local keystones = {}

    for _, realm in pairs(GetAutoCompleteRealms()) do
        for _, character in pairs(DataStore:GetCharacters(realm, THIS_ACCOUNT)) do
            --if DataStore:GetCharacterFaction(character) == THIS_FACTION then
                local keystone = GetKeystoneForCharacter(character)
                if keystone then
                    table.insert(keystones, {realm, DataStore:GetCharacterFaction(character), character, keystone})
                end
            --end
        end
    end

    return keystones
end

local function GetTooltipLines()
    local lines = {}

    local keystones = GetCharactersWithKeystones()
    if #keystones < 1 then
        table.insert(lines, "No Keystones")
        return lines
    end

    table.insert(lines, string.format("%d %s\n", #keystones, (#keystones == 1 and "Keystone" or "Keystones")))
    for _, data in pairs(keystones) do
        local realm, faction, character, keystone = unpack(data)
        local instanceMapId, level, isDepleted = GetKeystoneData(keystone)
        local instanceName = select(1, C_ChallengeMode_GetMapInfo(instanceMapId))

        if isDepleted then
            table.insert(lines, {left = GetCharacterDisplayName(realm, faction, character), right = string.format("|cff666666%s %s|r", instanceName, level)})
        else
            table.insert(lines, {left = GetCharacterDisplayName(realm, faction, character), right = string.format("|cffFFFFFF%s|r |cff00FF00%s|r", instanceName, level)})
        end
    end

    return lines
end

local ldb = LibStub:GetLibrary("LibDataBroker-1.1")

local broker = ldb:NewDataObject(addon, {
    type = "data source",
    text = "Keystones",
    OnClick = function() end,
})

function broker:OnTooltipShow()
    local lines = GetTooltipLines()

    for _, line in pairs(lines) do
        if type(line) == "string" then
            line = {left = line}
        end

        self:AddDoubleLine(line.left, line.right)
    end
end

function broker:OnEnter()
    GameTooltip:SetOwner(self, "ANCHOR_NONE")
    GameTooltip:SetPoint("BOTTOM", self, "BOTTOM")
    GameTooltip:ClearLines()
    broker.OnTooltipShow(GameTooltip)
    GameTooltip:Show()
end

function broker:OnLeave()
    GameTooltip:Hide()
end
